@@ -1,7 +1,7 @@
 /*
  This file is part of MOST.
 
- Copyright (c) 1991, 1999, 2002, 2005, 2006, 2007 John E. Davis
+ Copyright (c) 1991, 1999, 2002, 2005-2021, 2022 John E. Davis
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -15,7 +15,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ Mass Ave, Cambridge, MA 02139, USA.
 */
 #include "config.h"
 #include <stdio.h>
@@ -57,7 +57,7 @@ static void page_up_cmd (void)
    if (Most_Digit_Arg != (int *) NULL) n = *Most_Digit_Arg;
    n = Most_C_Line - n * (Most_Win->bot - Most_Win->top + 1);
    if (n < 1) n = 1;
-			  
+
    most_update_windows (n);
 }
 
@@ -116,7 +116,7 @@ static void tail_mode_cmd (void)
 
    do
      {
-	if (0 != most_read_file_dsc (-1))
+	if (0 != most_read_file_dsc (-1, 1))
 	  most_update_windows (-1);
 
 	most_message ("Most Tail Mode--  MOST keys are still active.", 0);
@@ -136,10 +136,14 @@ static void top_of_buffer_cmd(void)
 
 static void end_of_buffer_cmd(void)
 {
+   unsigned int count = 0;
+   /* This will loop forever if the file is changing too fast. */
    while ((Most_Buf->fd != -1)
-	  && (0 != most_read_file_dsc(-1)))
-     ;
+	  && (count < 5)
+	  && (0 != most_read_file_dsc(-1, 1)))
+     count++;
    most_update_windows (-1);
+   Most_Curs_Offset = (Most_Eob - Most_Beg);
 }
 #ifdef unix
 # include <signal.h>
@@ -199,7 +203,7 @@ static int read_integer (char *prompt, MOST_INT *n)
 {
    int status;
    Most_Mini_Buf[0] = 0;
-   
+
    status = most_read_from_minibuffer(prompt, NULL, (char *) Most_Mini_Buf, MOST_MINI_BUF_LEN);
    if (status < 0)
      return -1;
@@ -240,7 +244,7 @@ static void goto_percent_cmd(void)
      }
 
    if (n < 0) n = 0; else if (n > 100) n = 100;
-   if (Most_Buf->fd != -1) most_read_file_dsc(-1);
+   if (Most_Buf->fd != -1) most_read_file_dsc(-1, 1);
    pos = Most_Beg + (n * (Most_Eob - Most_Beg))/100;
    n = most_what_line(pos);
    most_update_windows (n);
@@ -325,24 +329,18 @@ static void search_cmd_dir (char *prompt, int dir)
 
 static void search_cmd(void)
 {
-   search_cmd_dir (
-#ifdef SLANG_REGEXP
-		   "Regexp Search: ",
-#else
-		   "Search: ",
-#endif
-		   1);
+   char *prompt = "Regexp-Search: ";
+   if (Most_Do_Regexp_Search == 0)
+     prompt += 7;
+   search_cmd_dir (prompt,1);
 }
 
 static void search_back_cmd(void)
 {
-   search_cmd_dir (
-#ifdef SLANG_REGEXP
-		   "Regexp Search Backwards: ",
-#else
-		   "Search Backwards: ",
-#endif
-		   -1);
+   char *prompt = "Regexp-Search Backwards: ";
+   if (Most_Do_Regexp_Search == 0)
+     prompt += 7;
+   search_cmd_dir (prompt, -1);
 }
 
 static void help_cmd(void)
@@ -591,6 +589,8 @@ void most_init_keymaps (void)
    SLkm_define_key ("\033[B", (FVOID_STAR) next_line_cmd, Most_Keymap);
    SLkm_define_key ("\033[C", (FVOID_STAR) column_right_cmd, Most_Keymap);
    SLkm_define_key ("\033[D", (FVOID_STAR) column_left_cmd, Most_Keymap);
+   SLkm_define_key ("\033[7~", (FVOID_STAR) top_of_buffer_cmd, Most_Keymap);
+   SLkm_define_key ("\033[8~", (FVOID_STAR) end_of_buffer_cmd, Most_Keymap);
    SLkm_define_key ("^@", (FVOID_STAR) set_mark_cmd, Most_Keymap);
 #else
    SLkm_define_key ("^@H", (FVOID_STAR) previous_line_cmd, Most_Keymap);
