@@ -1,7 +1,7 @@
 /*
  This file is part of MOST.
 
- Copyright (c) 1991, 1999, 2002, 2005 John E. Davis
+ Copyright (c) 1991, 1999, 2002, 2005-2021, 2022 John E. Davis
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -15,7 +15,7 @@
 
  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc., 675
- Mass Ave, Cambridge, MA 02139, USA. 
+ Mass Ave, Cambridge, MA 02139, USA.
 */
 #include "config.h"
 
@@ -48,10 +48,10 @@ char *Most_Global_Msg = "Press `Q' to quit, `H' for help, and SPACE to scroll.";
 
 Most_Window_Type *Most_Win;
 Most_Window_Type *Most_Top_Win;
-int Most_Top_Line;		       /* row number of top window */
-int Most_Curs_Row;
-int Most_Curs_Col;
-int Most_Column = 1;
+MOST_INT Most_Top_Line;		       /* row number of top window */
+MOST_INT Most_Curs_Row;
+MOST_INT Most_Curs_Col;
+MOST_INT Most_Column = 1;
 int Most_Restore_Width_To = 0;
 char Most_Mini_Buf[256];
 unsigned long Most_Curs_Offset;
@@ -112,7 +112,7 @@ void most_put_message ()
 	/* wait half a second */
 	if (SLang_input_pending (5))
 	  {
-	     (void) SLang_getkey ();
+	     (void) most_getkey ();
 	     SLKeyBoard_Quit = 0;
 	     SLang_set_error (0);
 	  }
@@ -138,7 +138,7 @@ static void rline_update (unsigned char *buf, int len, int col)
    SLsmg_refresh ();
 }
 #else
-static void rline_update (SLrline_Type *rli, char *prompt, 
+static void rline_update (SLrline_Type *rli, char *prompt,
 			  char *buf, unsigned int len, unsigned int point,
 			  VOID_STAR client_data)
 {
@@ -146,7 +146,7 @@ static void rline_update (SLrline_Type *rli, char *prompt,
 
    (void) client_data;
    (void) rli;
-   
+
    while (1)
      {
 	SLsmg_gotorc (SLtt_Screen_Rows - 1, 0);
@@ -182,7 +182,7 @@ static SLang_RLine_Info_Type  *init_readline (void)
    rli->buf_len = 255;
    rli->tab = 8;
    rli->dhscroll = 20;
-   rli->getkey = SLang_getkey;
+   rli->getkey = most_getkey;
    rli->tt_goto_column = NULL;
    rli->update_hook = rline_update;
 
@@ -197,7 +197,7 @@ static SLang_RLine_Info_Type  *init_readline (void)
 static char *SLrline_read_line (SLrline_Type *rli, char *prompt, unsigned int *lenp)
 {
    int i;
-   
+
    rli->prompt = prompt;
    i = SLang_read_line (rli);
    if (i < 0)
@@ -227,7 +227,10 @@ static SLang_RLine_Info_Type  *init_readline (void)
 {
    SLang_RLine_Info_Type *rli;
    unsigned int flags = SL_RLINE_BLINK_MATCH;
-   
+
+   if (Most_UTF8_Mode)
+     flags |= SL_RLINE_UTF8_MODE;
+
    if (NULL == (rli = SLrline_open (SLtt_Screen_Cols, flags)))
      return NULL;
 
@@ -236,7 +239,7 @@ static SLang_RLine_Info_Type  *init_readline (void)
 }
 #endif
 
-int most_read_from_minibuffer(char *prompt, char *what, unsigned int buflen)
+int most_read_from_minibuffer(char *prompt, char *stuff, char *what, unsigned int buflen)
 {
    int i = 0;
    char *buf;
@@ -257,13 +260,12 @@ int most_read_from_minibuffer(char *prompt, char *what, unsigned int buflen)
    most_select_minibuffer();
 
    /* do not use default.  The up arrow can always get it back. */
-   if ((what != NULL) 
-       && (*what) && (what != Most_Search_Str))
+   if (stuff != NULL)
      {
-	if (-1 == SLrline_set_line (Most_RLI, what))
+	if (-1 == SLrline_set_line (Most_RLI, stuff))
 	  return -1;
 
-	SLrline_set_point (Most_RLI, strlen (what));
+	SLrline_set_point (Most_RLI, strlen (stuff));
      }
 
    buf = SLrline_read_line (Most_RLI, prompt, &len);
@@ -284,7 +286,7 @@ int most_read_from_minibuffer(char *prompt, char *what, unsigned int buflen)
 
    if (i == -1) most_message ("Aborted!", 1);
 
-   return(i);
+   return i;
 }
 
 void most_clear_minibuffer()
@@ -302,10 +304,10 @@ void most_check_minibuffer ()
        put_message_1 (Most_Global_Msg);
 }
 
-static int get_scroll(int *line)
+static int get_scroll(MOST_INT *line)
 {
     /* line is the line we want at the top of the window if possible */
-   int dtop, dbot,n,top,bot,wsize;
+   MOST_INT dtop, dbot, n, top, bot, wsize;
 
    top = Most_Win->beg_line;
    wsize = Most_Win->bot - Most_Win->top; /* actually 1 less than window size */
@@ -372,15 +374,15 @@ static void check_dirty_flag (void)
    if (Most_Win->dirty_flag == 0) return;
 
    Most_Win->dirty_flag = 0;
-   /* 
+   /*
    Most_Num_Lines = most_count_lines (Most_Beg, Most_Eob);
     */
    Most_Win->n_lines = Most_Num_Lines;
 }
 
-void most_update_window (int line)
+void most_update_window (MOST_INT line)
 {
-   int save_line, save_col;
+   MOST_INT save_line, save_col;
    unsigned long save_offset;
 
    most_read_to_line (line);
@@ -413,10 +415,11 @@ void most_update_window (int line)
 /* updates current window as well as scroll lock ones */
 /* Although current window is update in an absolute fashion, scroll locked
    ones are updated in a relative fashion */
-void most_update_windows (int line)
+void most_update_windows (MOST_INT line)
 {
-   int dline,flg;
    Most_Window_Type *w;
+   MOST_INT dline;
+   int flg;
 
    if (line == -1)
      {
@@ -464,7 +467,7 @@ void most_redraw_window (void)
    r = Most_Win->top;
    most_goto_rc (r, 1);
 
-   most_display_line ();
+   most_display_line (1);
 
    while (n--)
      {
@@ -472,7 +475,7 @@ void most_redraw_window (void)
 	most_goto_rc (r, 1);
 
 	if (most_forward_line(1))
-	  most_display_line();
+	  most_display_line(0);
 	else
 	  SLsmg_erase_eol ();
      }
@@ -547,6 +550,11 @@ static Most_Window_Type *make_window(int r1,int r2)
 
 int Most_Display_Inited = 0;
 
+static void error_hook (char *msg)
+{
+   most_message (msg, 1);
+}
+
 void most_init_display ()
 {
    int h = SLtt_Screen_Rows;
@@ -565,6 +573,8 @@ void most_init_display ()
    else Most_Top_Win->prev->bot = SLtt_Screen_Rows - 2;
 
    SLsmg_init_smg ();
+   SLang_Error_Hook = error_hook;
+
    most_goto_rc (Most_Win->top,1);
    Most_Display_Inited = 1;
    /* SLsmg_refresh (); */
@@ -593,14 +603,15 @@ static void update_status1 (void)
    unsigned int num_chars;
    unsigned int field_width, info_len;
    unsigned char *eob;
-   int r, x;
+   int r;
+   MOST_INT x;
 
    eob = Most_Eob;
 #if MOST_HAS_MMAP
    if (Most_Buf->is_mmaped)
-     eob = Most_Beg + Most_Buf->mmap_size;
+     eob = Most_Beg + Most_Buf->mmap_file_size;
 #endif
-   
+
    if (eob == Most_Beg) x = 100;
    else
      {
@@ -614,7 +625,8 @@ static void update_status1 (void)
    /* for files with end of file above the bottom row (due to window manipulations) */
    if (x > 100) x = 100;
 
-   sprintf (info, "(%d,%d) %d%%", Most_C_Line, Most_Column, x);
+   sprintf (info, "(" MOST_INT_D_FMT "," MOST_INT_D_FMT ")" MOST_INT_D_FMT "%%",
+	    Most_C_Line, Most_Column, x);
 
    r = Most_Win->bot + 1;
    most_goto_rc (r,1);
@@ -627,10 +639,10 @@ static void update_status1 (void)
 
    num_chars = strlen (buf);
    SLsmg_write_nchars (buf, num_chars);
-   
+
    /* So far num_chars have been written out.  We would like to put the
     * line number information about info_len charcters from the RHS of the
-    * screen.  This leaves a field width of 
+    * screen.  This leaves a field width of
     * SLtt_Screen_Cols - (16 + num_chars) for the filename.
     */
    info_len = strlen (info);
@@ -687,7 +699,7 @@ void most_update_status (void)
 /* splits window-- no screen update, does not affect scrolling region */
 int most_split_window (void)
 {
-   Most_Window_Type *new, *old;
+   Most_Window_Type *neew;
    int b2,t2,b1, line;
 
    b2 = Most_Win->bot;
@@ -697,20 +709,19 @@ int most_split_window (void)
 
     /* line is top line of new window. */
    line = Most_Win->beg_line + t2 - Most_Win->top;
-   old = Most_Win;
    Most_Win->bot = b1;
-   new = make_window(t2,b2);
+   neew = make_window(t2,b2);
     /* add to ring */
-   Most_Win->next->prev = new;
-   new->next = Most_Win->next;
-   new->prev = Most_Win;
-   Most_Win->next = new;
+   Most_Win->next->prev = neew;
+   neew->next = Most_Win->next;
+   neew->prev = Most_Win;
+   Most_Win->next = neew;
 
-   new->beg_line = line;
-   new->buf = Most_Buf;
+   neew->beg_line = line;
+   neew->buf = Most_Buf;
 #if 0
     /* new window status line is at same position as old */
-   strcpy(new->status,Most_Win->status);
+   strcpy(neew->status,Most_Win->status);
 #endif
    return(1);
 }
@@ -863,16 +874,13 @@ void most_other_window(int n)
 /* kills window by moving lower window up */
 static void delete_as_top_window (void)
 {
-   int t1,t2,b1,b2;
-   t1 = Most_Win->top;
-   t2 = Most_Win->next->top;
-   b1 = Most_Win->bot;
-   b2 = Most_Win->next->bot;
+   MOST_INT t = Most_Win->top;
+
    Most_Win->prev->next = Most_Win->next;
    Most_Win->next->prev = Most_Win->prev;
 
    most_other_window(1);
-   Most_Win->top = t1;
+   Most_Win->top = t;
    most_redraw_window();
    most_update_status();
 }
@@ -892,7 +900,7 @@ void most_free_window_buffer (void)
      }
    b = w->buf;
    if (b == NULL) return;
-   
+
    most_close_buffer_file (b);
    Most_Win->buf = NULL;
    SLFREE(b);
@@ -924,7 +932,7 @@ void most_free_windows (void)
 
 void most_delete_window (void)
 {
-   int new_b, old_b;
+   int new_b;
    Most_Window_Type *w;
 
    w = Most_Win;
@@ -947,7 +955,6 @@ void most_delete_window (void)
 	return;
      }
 
-   old_b = Most_Win->top - 2;
    new_b = Most_Win->bot;
    most_other_window(-1);
    Most_Win->bot = new_b;
@@ -971,7 +978,7 @@ void most_delete_window (void)
 void most_redraw_display (void)
 {
    Most_Window_Type *w;
-   int n,t;
+   MOST_INT n, t;
 
    SLsmg_cls ();
 
@@ -993,13 +1000,13 @@ void most_redraw_display (void)
 
 	t = Most_Win->top;
 	most_goto_rc(t, 1);
-	most_display_line();
+	most_display_line(1);
 
 	while (n--)
 	  {
 	     t++;
 	     most_goto_rc (t, 1);
-	     if (most_forward_line(1)) most_display_line();
+	     if (most_forward_line(1)) most_display_line(0);
 	  }
 	Most_C_Line = Most_Win->beg_line;
 	Most_C_Offset = Most_Win->top_offset;
